/**
  *  \file client/chart.h
  *  \brief Starchart Drawing
  *
  *  (c) 2001-2006 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_CHART_H
#define PCC_V2_CLIENT_CHART_H

#include <SDL.h>
#include "cpluslib/smartptr.h"
#include "gfx/gfx.h"
#include "game/coord.h"
#include "game/global.h"
#include "ui/skin.h"

class GUniverse;

extern const GfxFillPattern IONSTORM_FILL, IONSTORM_DENSE_FILL;

//! Chart Options (what/how to display).
struct GChartOptions {
    int32 show;
    int32 fill;

    /** Starchart Option Bitflags */
    enum {
        co_Ion       = 1,
        co_Mine      = 2,
        co_Ufo       = 4,
        co_Sectors   = 8,
        co_Borders   = 16,
        co_Drawings  = 32,
        co_Selection = 64,
        co_Labels    = 128,
        co_Trails    = 256,

        co_Tristate  = co_Ion + co_Mine + co_Ufo,

        co_Default   = ~0U - co_Trails
    };
};

//! Chart Viewport.
class GChartViewport {
    GfxPoint origin_screen;
    GfxPoint center_screen;

    void drawMovingObject(GfxCanvas& can, GfxParameters& parm, GfxPoint pt,
                          int r, int speed, int heading, int32 f);
    
    void drawAux(GfxCanvas& can, GChartOptions& opt, GUniverse& trn,
                 const GChartBBox& bbox);
    void drawShipSelAndVectors(GfxCanvas& can, GUniverse& trn, int show);
    void drawPlanets(GfxCanvas& can, GUniverse& trn, int show);
    void drawShips(GfxCanvas& can, GUniverse& trn, int show);
    void drawSectors(GfxCanvas& can, int show);
    void drawMines(GfxCanvas& can, GUniverse& trn, int fill);
    void drawCross(GfxCanvas& can, GfxParameters& parm, GfxPoint pt);
    void drawIons(GfxCanvas& can, GUniverse& trn, int fill);
 public:
    GfxRect view;               /**< Screen positions. */
    GPoint center;              /**< Center coordinate of map. */
    int mult;                   /**< Zoom factor, numerator. */
    int divi;                   /**< Zoom factor, denominator. */
    int vec_id;                 /**< Id of ship whose vector is shown. */

    void init();
    void draw(GfxCanvas& can, GChartOptions& opt, GUniverse& trn,
              const GChartBBox& bbox);
    int scale(int n) const;
    int scaleClip(int n, int add) const;

    /** Convert game X coordinate to screen X coordinate. */
    int scaleX(int x) const
        { return origin_screen.x + scale(x); }
    /** Convert game Y coordinate to screen Y coordinate. */
    int scaleY(int y) const
        { return origin_screen.y - scale(y); }
    /** Convert game point to screen point. */
    GfxPoint scaleXY(GPoint pt) const
        { return GfxPoint(scaleX(pt.x), scaleY(pt.y)); }

    int unscale(int n) const;

    /** Convert screen X coordinate to game X coordinate. */
    int unscaleX(int x) const
        { return unscale(x - origin_screen.x); }
    /** Convert screen Y coordinate to game Y coordinate. */
    int unscaleY(int y) const
        { return unscale(origin_screen.y - y); }
    /** Convert screen point to game point. */
    GPoint unscaleXY(GfxPoint pt) const
        { return GPoint(unscaleX(pt.x), unscaleY(pt.y)); }

    /** Get screen coordinate of viewport center. This is the
        coordinate which \c center maps to. */
    const GfxPoint& getCenter() const
        { return center_screen; }

    bool zoomIn(GChartBBox& bbox);
    bool zoomOut(GChartBBox& bbox);
};

GChartOptions& getChartOpts(bool is_scanner, int mult, int divi);
UIColor::Color getPlayerColor(int player);

#endif
